#!/usr/bin/env python

import logging
import optparse
import os
import pathlib
import re
import shutil
import sys
import tempfile

import fitz
import osascript

from graffle2pdftex.omnigraffle import OmniGraffle

# graffle2pdftex_applescript = """
# set delayMultiplier to 1
# tell application "System Events"
# 	set omnigraffleIsOpen to ((bundle identifier of processes) contains "com.omnigroup.OmniGraffle7")
# end tell
#
# set omnigraffleDelay to 1
# if omnigraffleIsOpen is false then
# 	set omnigraffleDelay to 0
# end if
#
# set launchdelay to omnigraffleDelay
#
# tell application "OmniGraffle"
# 	activate
# end tell
# delay launchdelay
#
# --set destinationPath to POSIX path of alias ((path to home folder as text) & "OmniGraffle:PDF_Tex")
# tell application "System Events" to set frontmost of process "OmniGraffle" to true
# tell application "System Events" to click UI element "OmniGraffle" of list 1 of application process "Dock"
#
# delay launchdelay * delayMultiplier
#
# set OGCanvases to []
# set canvas_export_folders to []
# tell application "OmniGraffle"
# 	set _document to front document
# 	set originalFilepath to path of _document
# 	set _path to my removeExtension(originalFilepath)
# 	set basefolder to my removeLastPathComponent(originalFilepath)
# 	set filename to my getLastPathComponent(_path)
# 	set destinationPath to _path
# 	tell front document
# 		set myCanvases to (every canvas as list)
# 		repeat with i from 1 to (length of (myCanvases as list))
# 			set theName to (name of item i of myCanvases) as text
# 			set canvasExportPath to _path & "/" & filename & "_" & theName
# 			set OGCanvases to OGCanvases & theName
# 			set canvas_export_folders to canvas_export_folders & canvasExportPath
# 		end repeat
# 		--close
# 	end tell
# end tell
#
# (*
# do shell script "mkdir -p " & quoted form of _path
# set destinationDir to basefolder & "/tmp/"
# set destinationFilepath to destinationDir & filename & ".graffle"
#
# do shell script "mkdir -p " & quoted form of destinationDir & " && cp -rf " & quoted form of originalFilepath & " " & quoted form of destinationFilepath
# set theOPMLFilePosix to POSIX file destinationFilepath
#
# -- https://www.macscripter.net/t/converting-from-alias-to-path-path-to-alias/49846/2
# tell application "OmniGraffle"
# 	--do shell script "open -n " & quoted form of POSIX path of theOPMLFilePosix & " & "
# 	--log theOPMLFilePosix as alias as Unicode text
# 	set convertedDocument to open theOPMLFilePosix
# 	repeat until exists front document
# 		delay 0.1
# 	end repeat
# 	set _document to front document
# 	set _path to my removeExtension(path of _document)
# 	set filename to my getLastPathComponent(_path)
# 	tell front document
# 	end tell
# end tell
# *)
#
# set destinationFilepath to originalFilepath
# set theOPMLFilePosix to POSIX file destinationFilepath
#
# tell application "System Events"
# 	tell application process "OmniGraffle"
#
# 		repeat with i from 1 to (length of OGCanvases)
# 			set canvas to item i of OGCanvases
# 			--set outputPDFPath to destinationPath & "/" & filename & "_" & canvas & "/" & filename & "_" & canvas & ".pdf"
#
# 			click menu item canvas of menu 1 of menu item "Display Canvas" of menu 1 of menu bar item "View" of menu bar 1
# 			click menu item "Export to Pdf_Tex" of menu 1 of menu bar item "Automation" of menu bar 1
#
# 			repeat until exists sheet 1 of window 1
# 				delay 0.1
# 			end repeat
# 			tell sheet 1 of window 1
# 				keystroke "g" using {command down, shift down}
# 				repeat until exists sheet 1
# 					delay 0.1
# 				end repeat
# 				tell sheet 1
# 					keystroke destinationPath
# 					keystroke return
# 				end tell
# 				try
# 					repeat while exists sheet 1
# 						delay 0.1
# 					end repeat
# 				end try
# 				click UI element "Save"
# 				try
# 					click UI element "Replace" of sheet 1
# 				on error errorMessage
# 				end try
# 			end tell
#
# 			--log outputPDFPath
# 			--repeat until my fileExists(outputPDFPath)
# 			--	delay 0.1 -- wait for 0.1 second
# 			--end repeat
#
#
# 		end repeat
#
# 	end tell
# end tell
#
# -- https://www.macscripter.net/t/converting-from-alias-to-path-path-to-alias/49846/2
# tell application "OmniGraffle"
# 	tell front document
# 		close
# 	end tell
# end tell
#
# do shell script "rm -rf " & quoted form of destinationDir
# repeat with i from 1 to (length of (canvas_export_folders as list))
# 	set canvasExportFolder to item i of canvas_export_folders
# 	do shell script "cp -f " & quoted form of canvasExportFolder & "/*" & " " & quoted form of destinationPath & " && rm -rf " & quoted form of canvasExportFolder
# end repeat
#
# on removeExtension(theString)
# 	set lastPeriod to 0
# 	set n to count of characters of theString
# 	repeat with i from 1 to n
# 		if character i of theString is "." then
# 			set lastPeriod to i
# 		end if
# 	end repeat
#
# 	if lastPeriod is not 0 then
# 		set theString to characters 1 thru (lastPeriod - 1) of theString
# 		set theString to theString as text
# 	else
# 		--there isn't an extension
# 	end if
#
# 	return theString
# end removeExtension
#
# on getLastPathComponent(thePath)
# 	set tid to AppleScript's text item delimiters
# 	set AppleScript's text item delimiters to "/"
# 	set pathComponents to text items of thePath
# 	set AppleScript's text item delimiters to tid
# 	return last item of pathComponents
# end getLastPathComponent
#
#
# on removeLastPathComponent(thePath)
# 	set tid to AppleScript's text item delimiters
# 	set AppleScript's text item delimiters to "/"
# 	set pathComponents to text items of thePath
# 	set countOfComponents to count of pathComponents
# 	if countOfComponents > 1 then
# 		set newPath to text items 1 thru (countOfComponents - 1) of pathComponents as text
# 	else
# 		set newPath to ""
# 	end if
# 	set AppleScript's text item delimiters to tid
# 	return newPath
# end removeLastPathComponent
#
#
# on fileExists(filePath)
# 	try
# 		set fileRef to filePath as alias
# 		return true
# 	on error
# 		return false
# 	end try
# end fileExists
# """


def redact_text_from_pdf(input_pdf_path, output_pdf_path):
    document = fitz.open(input_pdf_path)

    for page_number in range(len(document)):
        page = document.load_page(page_number)

        # Extract the text and its details
        text_instances = page.get_text("dict")["blocks"]

        # Iterate through the text instances and replace with whitespaces
        for instance in text_instances:
            if hasattr(instance, "lines") or "lines" in instance:
                for line in instance["lines"]:
                    for span in line["spans"]:
                        text = span["text"]
                        # Calculate the number of whitespaces needed
                        # white_spaces = ' ' * len(text)
                        # Get the position of the text
                        rect = fitz.Rect(span["bbox"])
                        # Add a white space to replace the original text
                        # page.add_freetext(rect, white_spaces, fontsize=span["size"], color=(1, 1, 1))
                        page.add_redact_annot(rect)  # Mark the text for redaction

        # Apply redaction, effectively removing the text
        page.apply_redactions()


    document.save(output_pdf_path)

# https://stackoverflow.com/questions/4427542/how-to-do-sed-like-text-replace-with-python
def sed_inplace(filename, pattern, repl):
    '''
    Perform the pure-Python equivalent of in-place `sed` substitution: e.g.,
    `sed -i -e 's/'${pattern}'/'${repl}' "${filename}"`.
    '''
    # For efficiency, precompile the passed regular expression.
    pattern_compiled = re.compile(pattern)

    # For portability, NamedTemporaryFile() defaults to mode "w+b" (i.e., binary
    # writing with updating). This is usually a good thing. In this case,
    # however, binary writing imposes non-trivial encoding constraints trivially
    # resolved by switching to text writing. Let's do that.
    with tempfile.NamedTemporaryFile(mode='w', delete=False) as tmp_file:
        with open(filename) as src_file:
            for line in src_file:
                tmp_file.write(pattern_compiled.sub(repl, line))

    # Overwrite the original file with the munged temporary file in a
    # manner preserving file attributes (e.g., permissions).
    shutil.copystat(filename, tmp_file.name)
    shutil.move(tmp_file.name, filename)


# https://github.com/fikovnik/omnigraffle-export
def export(source, debug=False):
    # logging
    if debug:
        logging.basicConfig(level=logging.DEBUG)
    else:
        logging.basicConfig(level=logging.INFO, format='%(message)s')

    # check source
    if not os.access(source, os.R_OK):
        print("File: %s could not be opened for reading" % source, file=sys.stderr)
        sys.exit(1)

    filepath = pathlib.Path(source)
    destination = filepath.parent / filepath.stem
    destination.mkdir(parents=True, exist_ok=True)
    with tempfile.TemporaryDirectory() as temp_dir:
        tmpFilepath = pathlib.Path(temp_dir) / filepath.name
        tmpGrafflePath = pathlib.Path(temp_dir) / filepath.stem
        tmpGrafflePath.mkdir(parents=True, exist_ok=True)
        shutil.copyfile(str(filepath), str(tmpFilepath))
        og = OmniGraffle()
        schema = og.open(str(tmpFilepath))
        ogscriptPath = pathlib.Path(__file__).resolve().parent / "graffle2pdftex.applescript"
        osascript.osascript(str(ogscriptPath))
        # osascript.osascript(graffle2pdftex_applescript)

        # for file in tmpGrafflePath.rglob('*.pdf*'):
        #     shutil.copy(str(file), str(destination))

        for file in tmpGrafflePath.rglob('*.pdf'):
            redact_text_from_pdf(str(file), str(destination / file.name))

        for file in tmpGrafflePath.rglob('*.pdf_tex*'):
            shutil.copy(str(file), str(destination))


def main():
    usage = "Usage: %prog [options] <source> <target>"
    parser = optparse.OptionParser(usage=usage)

    parser.add_option('--debug', action='store_true', help='debug', dest='debug')
    (options, args) = parser.parse_args()

    if len(args) != 1:
        parser.print_help()
        sys.exit(1)
    (source,) = args

    export(source, options.debug)

if __name__ == '__main__':
    main()



import logging
import os
import time

import osascript as osascript
from appscript import k as keywordwrapper, app, ApplicationNotFoundError


# https://github.com/fikovnik/omnigraffle-export
class OmniGraffleSchema(object):
    """ A class that encapsulates an OmniGraffle schema file"""

    # attribute header in PDF document that contains the checksum
    PDF_CHECKSUM_ATTRIBUTE = 'OmnigraffleExportChecksum: '
    TIMEOUT = 5  # timeout waiting for export to be generated in seconds
    INTERVAL = 0.05  # interval at which to check for export completion (seconds)

    def __init__(self, og, doc):

        self.og = og
        self.doc = doc
        self.path = doc.path()

    def has_export_function(self):
        return tuple(list(map(int, self.og.version().split('.')))[:2]) >= (7, 8)

    def sandboxed(self):
        # real check using '/usr/bin/codesign --display --entitlements - /Applications/OmniGraffle.app'
        return self.og.version()[0] >= '6'

    def get_sandbox_path(self):
        version = self.og.version()[0]
        path = os.path.expanduser(OmniGraffle.SANDBOXED_DIR % version)

        if not os.path.exists(path):
            raise RuntimeError('OmniGraffle is sandboxed but missing sandbox path: %s' % path)

        return path

    def get_canvas_list(self):
        """
        Returns a list of names of all the canvases in the document
        """

        return [c.name() for c in self.doc.canvases()]

    def export(self, canvasname, fname, format='pdf', stripText=False):
        """
        Exports one canvas named `canvasname into `fname` using `format` format.
        """

        # canvas name
        assert canvasname and len(canvasname) > 0, 'canvasname is missing'

        # format
        if format not in OmniGraffleSchema.EXPORT_FORMATS:
            raise RuntimeError('Unknown format: %s' % format)

        export_format = OmniGraffleSchema.EXPORT_FORMATS[format]

        export_path = fname
        # Is OmniGraffle sandboxed?
        if self.sandboxed():
            export_path = self.get_sandbox_path() + os.path.basename(fname)
            logging.debug('OmniGraffle is sandboxed - exporting to: %s' % export_path)

        # StripText?
        if stripText == True:
            self.og.activate()
            osascript.osascript(" ".join(["tell application \"System Events\"", "\nclick menu item \"StripText\" of menu 1 of menu bar item \"Automation\" of menu bar 1 of application process \"OmniGraffle\"", "\nend tell"]))

         # find canvas
        canvas = [c for c in self.doc.canvases() if c.name() == canvasname]
        if len(canvas) == 1:
            canvas = canvas[0]
        else:
            raise RuntimeError('Canvas %s does not exist in %s' % (canvasname, self.doc.path()))

        # export
        self.og.windows.first().canvas.set(canvas)

        if self.has_export_function():
            # Omnigraffle 7.8+
            if (export_format == None):
                self.doc.export(scope=keywordwrapper.all_graphics, to=export_path)
            else:
                self.doc.export(as_=export_format, scope=keywordwrapper.all_graphics, to=export_path)

            # Wait for the exported file to finish generating - doc.export is asynchronous
            waited = 0
            while not os.path.exists(export_path) and waited < self.TIMEOUT:
                time.sleep(self.INTERVAL)
                waited += self.INTERVAL

        else:
            # Omnigraffle < 7.8
            self.og.current_export_settings.area_type.set(keywordwrapper.all_graphics)

            # FIXME: does this return something or throw something?
            if (export_format == None):
                self.doc.save(in_=export_path)
            else:
                self.doc.save(as_=export_format, in_=export_path)

        if self.sandboxed():
            os.rename(export_path, fname)
            logging.debug('OmniGraffle is sandboxed - moving %s to: %s' % (export_path, fname))

        logging.debug("Exported `%s' into `%s' as %s" % (canvasname, fname, format))

        # appscript.reference.CommandError: Command failed:
        # OSERROR: -50
        # MESSAGE: The document cannot be exported to the specified format.
        # COMMAND: app(u'/Applications/OmniGraffle Professional 5.app').documents[u'schemas.graffle'].save(in_=u'/Users/krikava/Documents/Research/Thesis/phd-manuscript/thesis/figures/mape-k.pdf', as_='Apple PDF pasteboard type')
        # raise RuntimeError('Failed to export canvas: %s to %s' % (canvasname, fname))

    def active_canvas_name(self):
        """
        Returns an active canvas name. The canvas that is currently selected in the the active OmniGraffle window.
        """
        window = self.og.windows.first()
        canvas = window.canvas()
        return canvas.name()

class OmniGraffle(object):

    SANDBOXED_DIR = '~/Library/Containers/com.omnigroup.OmniGraffle%s/Data/'

    def __init__(self):
        names = ['OmniGraffle 5.app', 'OmniGraffle Professional 5.app', 'OmniGraffle']
        self.og = None
        for name in names:
            try:
                self.og = app(name)
                break
            except (ApplicationNotFoundError):
                continue

        if self.og == None:
            raise RuntimeError('Unable to connect to OmniGraffle (%s)' % ', '.join(names))

    def active_document(self):
        self.og.activate()
        window = self.og.windows.first()
        doc = window.document()

        if doc == None:
            # this means that the active window is actually an OmniGraffle document window
            # it can be for example the license window
            # we cannot do anything
            return None

        fname = doc.path()
        if fname == None:
            fname = "Untitled"
        logging.debug('Active OmniGraffle file: ' + fname)

        return OmniGraffleSchema(self.og, doc)

    def open(self, fname):
        fname = os.path.abspath(fname)
        if not os.path.isfile(fname) and \
                not os.path.isfile(os.path.join(fname, "data.plist")):
            raise ValueError('File: %s does not exists' % fname)

        fname = os.path.abspath(fname)
        self.og.activate()

        # adhoc fix for https://github.com/fikovnik/omnigraffle-export/issues/23
        # apparently the process is sandboxed and cannot access the file
        # 16/03/2015 13:01:54.000 kernel[0]: Sandbox: OmniGraffle(66840) deny file-read-data test.graffle
        # therefore we first try to open it manually

        import subprocess
        subprocess.call(['open',fname])

        window = self.og.windows.first()
        # doc = window.document()
        doc = self.og.open(fname)

        logging.debug('Opened OmniGraffle file: ' + fname)

        return OmniGraffleSchema(self.og, doc)



import os
import pathlib
import shutil

from setuptools import setup, find_packages
from setuptools.command.install import install
# from pkg_resources import resource_filename

def read(fname):
    return open(os.path.join(os.path.dirname(__file__), fname)).read()

class CustomInstall(install):
    def run(self):
        # Run the standard install process
        install.run(self)

        self.copy_javascript_file()

    def copy_javascript_file(self):
        # Get the home directory
        home_directory = pathlib.Path.home()

        # Get the location of the installed package
        # localPackageInstallerPath = pathlib.Path(os.path.abspath(__file__)).parent / "graffle2pdftex"
        # binPath = pathlib.Path(self.install_scripts)
        sitePackagesPath = pathlib.Path(self.install_lib) / "graffle2pdftex"

        # Delete Excess & Junk Data
        thankyousetuptools_bullshit_folder = pathlib.Path(self.install_data) / "graffle2pdftex"
        shutil.rmtree(thankyousetuptools_bullshit_folder)

        # Path to the OmniGraffle Automation JS file
        src = sitePackagesPath / "graffle2pdftex.omnijs"
        dest = home_directory / "Library/Containers/com.omnigroup.OmniGraffle7/Data/Library/Application Support/Plug-Ins/"
        dest.mkdir(parents=True, exist_ok=True)
        shutil.copy(src, dest)

setup(
    name="graffle2pdftex",
    version="1.0.0",
    # packages=find_packages(include=['graffle2pdftex', 'graffle2pdftex.*']),
    include_package_data=True,
    packages=[*find_packages(include=["graffle2pdftex"])],
    # scripts=['graffle2pdftex/graffle2pdftex.applescript', 'graffle2pdftex/graffle2pdftex.omnijs'],
    data_files=[('graffle2pdftex', ['graffle2pdftex/graffle2pdftex.applescript', 'graffle2pdftex/graffle2pdftex.omnijs'])],
    cmdclass={
        'install': CustomInstall,
        'develop': CustomInstall,
    },
    install_requires=['appscript', 'pyobjc', "PyMuPDF"],
    author="Peter El-Jiz",
    author_email="peter.eljiz@gmail.com",
    description="A command line utility that exports omnigraffle canvases files to pdf_tex.",
    long_description=read("README.md"),
    keywords="graffle2pdftex",
    url="https://github.com/peterjiz/graffle2pdftex",
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        'Operating System :: MacOS :: MacOS X',
        "Environment :: Console",
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3.9",
        "Topic :: Utilities"
    ],
    entry_points={
        'console_scripts': [
            'graffle2pdftex = graffle2pdftex.graffle2pdftex:main',
        ],
    },
    zip_safe=True,
)
